%------------------------------------------------------------------------------%
% This file runs static optimization and joint analysis using our throcolumbar spine model to
% obtain muscle forces and joint reaction forces. These results are saved
% in an excel file format.
% We assume you have done all scaling to your model (either from imaging
% data or using marker sets, etc). Also, we assume that you have access to your
% motion and external force files (in the following format: either .mot or
% .sto). We provide 4 example motion and external force files to use in the
% tutorial.
% Authors: Alex Bruno, Dennis Anderson, Hossein Mokhtarzadeh, Katelyn Burkhart
% Updated for OpenSim Website, Hossein Mokhtarzadeh and Katelyn Burkhart 16 Oct 2017 BIDMC, HMS,
% @Bouxsein and Anderson Labs, Harvard Medical School and BIDMC.
%------------------------------------------------------------------------------%
%% Directory that may need to change:
FileDirectory = 'C:\OpenSim 3.3\Models\Thoracolumbar_Spine_Model\';% change to your file directory
import org.opensim.modeling.*
cd([FileDirectory])

%% Base model changes, make changes to your model
Models = {'Male_Thoracolumbar_Spine_Model.osim'};
%% The activities here are for this example. Any .trc (motion files) and then .mot (external forces) can be applied.
Activities = {'1','2','3','4'}; % numbers corresponding to posture motion files
%% Load external load and motion files.
for j = 1:4 % number of activities
    switch j
        case 1 % Neutral Standing
            disp('Standing')
            motionpath = strcat(FileDirectory, 'Motions\', 'NMB_Motion', Activities(j), '.mot');
            ExForce = strcat('NMB_ExternalForce', Activities(j), '.mot');
        case 2 % Axial Twist 30 degrees, elbows bent 90 degrees, with 10kg each hand
            disp('Axial Twist 30 with 10kg each hand')
            motionpath = strcat(FileDirectory, 'Motions\', 'NMB_Motion', Activities(j), '.mot');
            ExForce = strcat('NMB_ExternalForce', Activities(j), '.mot');
        case 3 % 30 Degree Trunk flexion, arms at sides, 5kg each hand
            disp('Flex 30, arms down, 5kg each hand')
            motionpath = strcat(FileDirectory, 'Motions\', 'NMB_Motion', Activities(j), '.mot');
            ExForce = strcat('NMB_ExternalForce', Activities(j), '.mot');
        case 4 % Pushing a shopping cart posture
            disp('Pushing forward against 50N each hand, elbows bent 90')
            motionpath = strcat(FileDirectory, 'Motions\', 'NMB_Motion', Activities(j), '.mot');
            ExForce = strcat('NMB_ExternalForce', Activities(j), '.mot');
        otherwise %Unlisted Activity
            disp('Unlisted Activity')
            motionpath = strcat(FileDirectory, 'Motions\', 'NMB_Motion', Activities(j), '.mot');
            ExForce = strcat('NMB_ExternalForce', Activities(j), '.mot');
    end
    %% results directories
    ResultDir = strcat(int2str(j),'\');
    %% Prepare for results files by setting all paths and making new directories
    TutorialDir = strcat(FileDirectory,'ThoracolumbarSpine_Tutorial_Male\'); % Directory where tutorial results are located
    extloadpath = [FileDirectory 'External Forces\' char(ExForce)];
    modelpath = [FileDirectory char(Models)];
    resultpath = [TutorialDir char(ResultDir)];
    mkdir(resultpath)
    cd([FileDirectory]);
    %% Analyze Motion Data
    % Perform Static Optimization
    Do_StatOpAnalysis(modelpath,motionpath,extloadpath,resultpath,FileDirectory)
    % Perform Joint Reaction Analysis
    Do_JntReactAnalysis(modelpath,motionpath,resultpath)
    % Get Vertebral Compressive Load
    [VertComp, VertAPShear, VertMLShear] = Get_VertLoad(resultpath);
    filename = [resultpath 'VertebralLoad.xlsx'];
    VComp = 'Compressive Load';
    APShear = 'AP Shear';
    MLShear = 'ML Shear';
    xlswrite(filename,VertComp,VComp)% write vertical compressive forces
    xlswrite(filename,VertAPShear,APShear)%AP shear forces
    xlswrite(filename,VertMLShear,MLShear)%ML shear forces
    copyfile([FileDirectory 'out.log'],resultpath); % Copies the out.log file which states if static optimization succesfully found a solution
end

%% Plotting the Spine Loading Results
cd(FileDirectory);

for i=1:4 % To compile forces from all activities into 1 variable per force direction
    LoadFile = strcat(TutorialDir,int2str(i),'\VertebralLoad.xlsx');
    resultpath = strcat(TutorialDir,int2str(i));
    cd(resultpath);
    CompData(i,:) = xlsread(LoadFile,'Compressive Load');
    APShearData(i,:) = xlsread(LoadFile,'AP Shear');
    MLShearData(i,:) = xlsread(LoadFile,'ML Shear');
end

% Plot Spine Compressive Forces
for i=1:4 
    
    subplot(2,2,i);
    plot(CompData(i,:),'LineWidth',2);
    xlabels = {'T1','T2','T3','T4','T5','T6','T7','T8','T9','T10','T11','T12','L1','L2','L3','L4','L5'};
    set(gca, 'Xtick', 1:17, 'XtickLabel', xlabels);
    ylabel('Compressive Spine Loading (N)');
    if i==1
        title('Standing')
    elseif i==2
        title('Axial Twist with 10kg')
    elseif i==3
        title('Flex 30 with 5kg')
    elseif i==4
        title('Pushing a Shopping Cart')
    end
    hold on
    
end

% % Plot Spine AP-Shear Forces
% figure;
% for i=1:4 
%     
%     subplot(2,2,i);
%     plot(APShearData(i,:),'LineWidth',2,'Color','g');
%     xlabels = {'T1','T2','T3','T4','T5','T6','T7','T8','T9','T10','T11','T12','L1','L2','L3','L4','L5'};
%     set(gca, 'Xtick', 1:17, 'XtickLabel', xlabels);
%     ylabel('AP Shear Spine Loading (N)');
%     if i==1
%         title('Standing')
%     elseif i==2
%         title('Axial Twist with 10kg')
%     elseif i==3
%         title('Flex 30 with 5kg')
%     elseif i==4
%         title('Pushing a Shopping Cart')
%     end
%     hold on
%     
% end
% 
% % Plot Spine ML-Shear Forces
% figure;
% for i=1:4 
%     
%     subplot(2,2,i);
%     plot(MLShearData(i,:),'LineWidth',2,'Color','m');
%     xlabels = {'T1','T2','T3','T4','T5','T6','T7','T8','T9','T10','T11','T12','L1','L2','L3','L4','L5'};
%     set(gca, 'Xtick', 1:17, 'XtickLabel', xlabels);
%     ylabel('ML Shear Spine Loading (N)');
%     if i==1
%         title('Standing')
%     elseif i==2
%         title('Axial Twist with 10kg')
%     elseif i==3
%         title('Flex 30 with 5kg')
%     elseif i==4
%         title('Pushing a Shopping Cart')
%     end
%     hold on
%     
% end
